#!/usr/bin/env python
'''
Report whether a login is valid. This is very useful for scripts.

It works like this:

   $ ./check_login.py <username> <password>

If no arguments are are given, the user is prompted.

   $ ./check_login.py
   Username: <username>
   Password: <password>

If the login is valid it prints "valid" and exits with status code 0.

If the login is not valid it prints "invalid" and exits with status
code 1.

Here are some example runs:

   $ ./check_login.py bigbob foobar
   valid
   $ echo $?
   0

   $ ./check_login.py bigbob foobarspam
   invalid
   $ echo $?
   1

   $ ./check_login.py
   Username: bigbob
   Password: <hidden - foobar>
   valid

   $ printf "bigbob\nfoobar\n" | ./check_login.py
'''
import argparse
import getpass
import os
import pexpect
import select
import sys
import time

def getopts():
    '''
    Get the command line options.
    '''
    this = os.path.basename(sys.argv[0])
    description = ('description:%s' % '\n  '.join(__doc__.split('\n')))
    rawd = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(formatter_class=rawd,
                                     description=description)

    parser.add_argument('-q', '--quiet',
                        action='store_true',
                        help='do not print valid or invalid, just set the exit code')

    parser.add_argument('-v', '--verbose',
                        action='count',
                        help='level of verbosity')

    # Display the version number and exit.
    parser.add_argument('-V', '--version',
                        action='version',
                        version='%(prog)s - v1.0')

    # Positional arguments at the end.
    parser.add_argument('LOGIN',
                        nargs="*",
                        help='[USERNAME [PASSWORD]]')

    opts = parser.parse_args()
    return opts


def main():
    '''
    main
    '''
    opts = getopts()

    # Need to check whether there is any pending input on stdin
    # to avoid hanging.
    lines = sys.stdin.readlines() if select.select([sys.stdin,],[],[],0.0)[0] else []
    if len(lines) > 1:
        # Read from a pipe.
        username = lines[0]
        password = lines[1]
    else:
        username = raw_input('Username: ') if len(opts.LOGIN) < 1 else opts.LOGIN[0]
        password = getpass.getpass('Password: ') if len(opts.LOGIN) < 2 else opts.LOGIN[1]
    
    # Figure out whether the login is valid by trying to "su"
    # and execute a simple command.
    ok = False
    child = pexpect.spawn('su -l ' + username, timeout=5)
    child.setecho(False)
    try:
        time.sleep(0.1)  # pause for a bit, like a human
        child.expect(r'[Pp]assword:')
        child.sendline(password)
        
        time.sleep(0.1)
        child.sendline('echo foobarspamblatz')
        child.expect(r'foobarspamblatz')
        
        time.sleep(0.1)
        child.sendline('exit 0')
        
        ok = True
    except pexpect.EOF as exc1:
        pass
    except pexpect.TIMEOUT as exc2:
        pass
    except OSError as exc3:
        pass
        
    if ok:
        if not opts.quiet:
            print('valid')
        sys.exit(0)

    if not opts.quiet:
        print('invalid')
    sys.exit(1)

if __name__ == '__main__':
    main()
